// Copyright (C) 2002 Christian Weis

// own includes
#include "ErrHandler.h"

// compiler includes
#define WIN32_LEAN_AND_MEAN
#define VC_EXTRALEAN
#define WIN32_EXTRALEAN

#include <windows.h>
#include <windowsx.h>
#include <string.h>

// history init string
#define STR_HISTORY "Exception History: "

enum {HISTORY = 1 << 10}; // history zstring len

// history zstring
static char history [HISTORY] = STR_HISTORY;
static bool disabled = 0;
static bool first_func = !0;

// disables err handling
void apiDisableErrHandler () {PROTECTED (disabled = !0;)}

// handles any exception with func call stack
void apiHandleErr (const char *pe, const char *pfunc)
{
	PROTECTED
	(
		if (!disabled)
		{
			if (pe && *pe)
			// add exception
			{
				strncat (history, pe, HISTORY);
				strncat (history, "\n\n", HISTORY);
			}

			if (pfunc && *pfunc)
			// add func
			{
				if (!first_func) strncat (history, " -> ", HISTORY);
				strncat (history, pfunc, HISTORY);
				first_func = 0;
			}
		}
	)
}

// shows err and func call stack
void apiShowErr (const char *pcap)
{
	PROTECTED
	(
		PROTECTED (OutputDebugString (history);)
		PROTECTED
		(
			const char *pc = pcap && *pcap ? pcap : "Exception detected";
			MessageBox (0, history, pc, MB_OK | MB_ICONERROR | MB_TASKMODAL);
		)

		// re-init err handler
		strncpy (history, STR_HISTORY, HISTORY);
		disabled = 0;
		first_func = !0;
	)
}